//Basis++.cpp the implementation of a basis

#define BUILDING_BASIS_PLUSPLUS_H 1
#include "Basis++.h"

//=====================================================================
//constructors:

basisf::basisf()
{
  e = vectorf(1.0f,0.0f,0.0f);
  f = vectorf(0.0f,1.0f,0.0f);
  g = vectorf(0.0f,0.0f,1.0f);
  _ortho = true; _unit = true;
};

basisf::basisf(vectorf a, vectorf b, vectorf c)
{
  //make sure the vectors are not linearly independent.
  //if not, then set them to their respective components.
  
  if (snorm(a) == 0.0f) {e = vectorf(1.0f, 0.0f, 0.0f); f = b;}
  else if (snorm(a & b) == 0.0f) {e = a; f = vectorf(0.0f, 1.0f, 0.0f);}
  else {e = a; f = b;}; 
  
  if (snorm(e & f) == 0.0f) {f = vectorf(0.0f, 0.0f, 1.0f);};
  if (snorm(e & f) == 0.0f) {f = vectorf(0.0f, 1.0f, 0.0f);};
  
  if ((e & f) * c == 0.0f) {g = e & f;}
  else {g = c;};
  
  //check orthogonality
  _ortho = (e * f == 0.0f) && (e * g == 0.0f) && (f * g == 0.0f);
  
  //check if unitary
  _unit = (snorm(e) == 0.0f) && (snorm(f) == 0.0f) && (snorm(g) == 0.0f);
  
  //check handedness
  _right = ((e & f) * g > 0.0f);
};

basisf::basisf(basisf *lhs)
{e = lhs->e; f = lhs->f; g = lhs->g; _ortho = lhs->_ortho; _unit = lhs->_unit;};

//=====================================================================
//replacement functions

void basisf::replacee(vectorf lhs)
{
  e = lhs;
  //make sure the vector is not parallel to f or g
  if      (snorm(lhs & f) == 0) {f = e & g;}
  else if (snorm(lhs & g) == 0) {g = e & f;};
    
  //check orthogonality
  _ortho = (e * f == 0.0f) && (e * g == 0.0f) && (f * g == 0.0f);
  
  //check if unitary
  _unit = (norm(e) == 0.0f) && (norm(f) == 0.0f) && (norm(g) == 0.0f);
  
  //check handedness
  _right = ((e & f) * g > 0.0f);
};

void basisf::replacef(vectorf lhs)
{
  f = lhs;
  
  //make sure the vector is not parallel to e or g
  if      (snorm(lhs & e) == 0) {e = f & g;}
  else if (snorm(lhs & g) == 0) {g = e & f;};
    
  //check orthogonality
  _ortho = (e * f == 0.0f) && (e * g == 0.0f) && (f * g == 0.0f);
  
  //check if unitary
  _unit = (snorm(e) == 0.0f) && (snorm(f) == 0.0f) && (snorm(g) == 0.0f);
  
  //check handedness
  _right = ((e & f) * g > 0.0f); 
};

void basisf::replaceg(vectorf lhs)
{
  g = lhs;
  //make sure the vector is not parallel to e or f
  if      (snorm(lhs & e) == 0) {e = f & g;}
  else if (snorm(lhs & f) == 0) {f = e & g;};
    
  //check orthogonality
  _ortho = (e * f == 0.0f) && (e * g == 0.0f) && (f * g == 0.0f);
  
  //check if unitary
  _unit = (snorm(e) == 0.0f) && (snorm(f) == 0.0f) && (snorm(g) == 0.0f);
  
  //check handedness
  _right = ((e & f) * g > 0.0f); 
};

//=======================================================================
//state values

bool basisf::orthogonal() {return _ortho;};
bool basisf::unitary() {return _unit;};
bool basisf::righthanded() {return _right;};

//=======================================================================
//modifications

void basisf::orthogonalize()
{
  if (e * f != 0.0f) {f = f - (e << f);};
  if ((e & f) * g != 0.0f) {g =  (e & f) << g;};
  _ortho = true;
};

void basisf::unitize()
{
  e = unit(e); f = unit(f); g = unit(g);
};

void basisf::dextrize()
{
  if ((e & f) * g < 0) {g = -g;};
};

//=======================================================================
//Basis/vector functions:

vectorf tempf2;
float fptemp2;

vectorf operator <<(vectorf lhs, basisf rhs)
{
  tempf2.x = (lhs.x * rhs.e.x + lhs.y * rhs.f.x + lhs.z * rhs.g.x);
  tempf2.y = (lhs.x * rhs.e.y + lhs.y * rhs.f.y + lhs.z * rhs.g.y);
  tempf2.z = (lhs.x * rhs.e.z + lhs.y * rhs.f.z + lhs.z * rhs.g.z);
  return tempf2;
};

vectorf operator >>(vectorf lhs, basisf rhs)
{
  //find the determinant.
  fptemp2 = rhs.e * (rhs.f & rhs.g);
  tempf2.x = (rhs.e.x + rhs.f.x + rhs.g.x) * (lhs.x) / fptemp2;
  tempf2.y = (rhs.e.y + rhs.f.y + rhs.g.y) * (lhs.y) / fptemp2;
  tempf2.z = (rhs.e.z + rhs.f.z + rhs.g.z) * (lhs.z) / fptemp2;
  return tempf2;
};

bool operator <=(vectorf &lhs, basisf rhs)
{
  tempf2.x = (lhs.x * rhs.e.x + lhs.y * rhs.f.x + lhs.z * rhs.g.x);
  tempf2.y = (lhs.x * rhs.e.y + lhs.y * rhs.f.y + lhs.z * rhs.g.y);
  tempf2.z = (lhs.x * rhs.e.z + lhs.y * rhs.f.z + lhs.z * rhs.g.z);
  lhs = tempf2;
  return true;
};