#define BUILDING_PRIMITIVE_PLUSPLUS_H 1

#include <Message.h>
#include "Primitive++.h"
#include <TypeConstants.h>
#include <string.h>

status_t primitivef::Archive(BMessage *msg_archive, bool deep) const
{
  msg_archive->AddString("class","primitivef");
  return B_OK;
};

quadf::quadf()
{
  vertices[0].x = 0.0f;		vertices[0].y =  0.0f;		vertices[0].z = 0.0f;
  vertices[1].x = 1.0f;		vertices[1].y =  0.0f;		vertices[1].z = 0.0f;
  vertices[2].x = 1.0f;		vertices[2].y =  1.0f;		vertices[2].z = 0.0f;
  vertices[3].x = 0.0f;		vertices[3].y =  1.0f;		vertices[3].z = 0.0f;
};

quadf::quadf(vectorf v1, vectorf v2, vectorf v3, vectorf v4)
{
  vertices[0] = v1; vertices[1] = v2; vertices[2] = v3; vertices[3] = v4;
};

quadf::quadf(quadf *source)
{
  for(int index = 0; index < 4; index++)
    vertices[index] = source->vertices[index];
};

void quadf::transform(basisf source)
{
  for(int index = 0; index < 4; index++)
    vertices[index] <= source;
};

void quadf::translate(vectorf displacement)
{
  for(int index = 0; index < 4; index++)
    vertices[index] += displacement;
};

BArchivable *quadf::Instantiate(BMessage *msg_create)
{
  return new quadf(msg_create);
};

status_t quadf::Archive(BMessage *msg_sent, bool deep) const
{
  primitivef::Archive(msg_sent, deep);
    msg_sent->AddString("GType","QUAD");
    msg_sent->AddData("Coords",B_RAW_TYPE,&vertices,sizeof(vectorf) * 4);
    msg_sent->AddString("class","quadf");
  return B_OK;
};

quadf::quadf(BMessage *msg_data)
{
  ssize_t temp; const void *tempptr;
  msg_data->FindData("Coords",B_RAW_TYPE,&tempptr,&temp);
  memcpy(&vertices, tempptr, sizeof(vectorf) * 4);
};

void quadf::draw(){};
//This should be overloaded later on.

//====================================================================

trianglef::trianglef()
{
  vertices[0].x = 0.0f;		vertices[0].y =  0.0f;		vertices[0].z = 0.0f;
  vertices[1].x = 1.0f;		vertices[1].y =  0.0f;		vertices[1].z = 0.0f;
  vertices[2].x = 1.0f;		vertices[2].y =  1.0f;		vertices[2].z = 0.0f;
};

trianglef::trianglef(vectorf v1, vectorf v2, vectorf v3)
{
  vertices[0] = v1; vertices[1] = v2; vertices[2] = v3;
};

trianglef::trianglef(trianglef *source)
{
  for(int index = 0; index < 3; index++)
    vertices[index] = source->vertices[index];
};

void trianglef::transform(basisf source)
{
  for(int index = 0; index < 3; index++)
    vertices[index] <= source;
};

void trianglef::translate(vectorf displacement)
{
  for(int index = 0; index < 3; index++)
    vertices[index] += displacement;
};

BArchivable *trianglef::Instantiate(BMessage *msg_create)
{
  return new trianglef(msg_create);
};

status_t trianglef::Archive(BMessage *msg_sent, bool deep)const
{
  primitivef::Archive(msg_sent, deep);
    msg_sent->AddString("GType","TRIANGLE");
    msg_sent->AddData("Coords",B_RAW_TYPE,&vertices,sizeof(vectorf) * 3);
    msg_sent->AddString("class","trianglef");
  return B_OK;
};

trianglef::trianglef(BMessage *msg_data)
{
  ssize_t temp; const void *tempptr;
  msg_data->FindData("Coords",B_RAW_TYPE,&tempptr,&temp);
  memcpy(&vertices, tempptr, sizeof(vectorf) * 3);
};

void trianglef::draw(){};
//This should be overloaded later on as well.

pointf::pointf(){location.x = 0.0f; location.y = 0.0f; location.z = 0.0f;};
pointf::pointf(vectorf source){location = source;};
pointf::pointf(pointf *source){location = source->location;};

void pointf::transform(basisf source)
{
  location <= source;
};

void pointf::translate(vectorf displacement)
{
  location += displacement;
};

BArchivable *pointf::Instantiate(BMessage *msg_create)
{
  return new pointf(msg_create);
};

status_t pointf::Archive(BMessage *msg_sent, bool deep) const
{
  primitivef::Archive(msg_sent, deep);
    msg_sent->AddString("GType","POINT");
    msg_sent->AddData("Coords",B_RAW_TYPE,&location, sizeof(vectorf));
    msg_sent->AddString("class","pointf");
  return B_OK;
};

pointf::pointf(BMessage *msg_data)
{
  ssize_t temp;
  const void *tempptr;
  msg_data->FindData("Coords",B_RAW_TYPE, &tempptr, &temp);
  memcpy(&location, tempptr, sizeof(vectorf));
};

void pointf::draw(){};