#define BUILDING_VECTOR_PLUSPLUS_H 1
#include "Vector++.h"

vectorf tempf;
vectori tempi;
float fptemp;

//====================================================================
//constructors

vectorf::vectorf(){ x = 0; y = 0; z = 0;};
vectori::vectori(){ x = 0; y = 0; z = 0;};

vectorf::vectorf(float a, float b, float c){x = a; y = b; z = c;};
vectori::vectori(long int a, long int b, long int c){x = a; y = b; z = c;};

vectorf::vectorf(vectorf *term){x = term->x; y = term->y; z = term->z;};
vectori::vectori(vectori *term){x = term->x; y = term->y; z = term->z;};

//====================================================================
//sums

vectorf operator +(vectorf lhs, vectorf rhs)
{
  tempf.x = lhs.x + rhs.x;
  tempf.y = lhs.y + rhs.y;
  tempf.z = lhs.z + rhs.z;
  return tempf;
};

vectori operator +(vectori lhs, vectori rhs)
{
  tempi.x = lhs.x + rhs.x;
  tempi.y = lhs.y + rhs.y;
  tempi.z = lhs.z + rhs.z;
  return tempi;
};

bool operator +=(vectorf &lhs, vectorf rhs)
{
  lhs.x += rhs.x;
  lhs.y += rhs.y;
  lhs.z += rhs.z;
  return true;
};

bool operator +=(vectori &lhs, vectori rhs)
{
  lhs.x += rhs.x;
  lhs.y += rhs.y;
  lhs.z += rhs.z;
  return true;
};

//======================================================================
//differences

vectorf operator -(vectorf lhs, vectorf rhs)
{
  tempf.x = lhs.x - rhs.x;
  tempf.y = lhs.y - rhs.y;
  tempf.z = lhs.z - rhs.z;
  return tempf;
};

vectori operator -(vectori lhs, vectori rhs)
{
  tempi.x = lhs.x - rhs.x;
  tempi.y = lhs.y - rhs.y;
  tempi.z = lhs.z - rhs.z;
  return tempi;
};

bool operator -=(vectorf &lhs, vectorf rhs)
{
  lhs.x -= rhs.x;
  lhs.y -= rhs.y;
  lhs.z -= rhs.z;
  return true;
};

bool operator -=(vectori &lhs, vectori rhs)
{
  lhs.x -= rhs.x;
  lhs.y -= rhs.y;
  lhs.z -= rhs.z;
  return true;
};

//=====================================================================
//unaries
vectorf operator +(vectorf rhs){return rhs;};
vectori operator +(vectori rhs){return rhs;};

vectorf operator -(vectorf rhs)
{
  tempf.x = - rhs.x;
  tempf.y = - rhs.y;
  tempf.z = - rhs.z;
  return tempf;
};

vectori operator -(vectori rhs)
{
  tempi.x = -rhs.x;
  tempi.y = -rhs.y;
  tempi.z = -rhs.z;
  return tempi;
};

//======================================================================
//dot-products

float operator *(vectorf lhs, vectorf rhs)
{
  return (lhs.x * rhs.x) + (lhs.y * rhs.y) + (lhs.z * rhs.z);
};

long int operator *(vectori lhs, vectori rhs)
{
  return (lhs.x * rhs.x) + (lhs.y * rhs.y) + (lhs.z * rhs.z);
};


//======================================================================
//scalar products

vectorf operator *(vectorf lhs, float rhs)
{
  tempf.x = lhs.x * rhs;
  tempf.y = lhs.y * rhs;
  tempf.z = lhs.z * rhs;
  return tempf;
};

vectorf operator *(float lhs, vectorf rhs)
{
  tempf.x = lhs * rhs.x;
  tempf.y = lhs * rhs.y;
  tempf.z = lhs * rhs.z;
  return tempf;
};

vectori operator *(vectori lhs, long int rhs)
{
  tempi.x = lhs.x * rhs;
  tempi.y = lhs.y * rhs;
  tempi.z = lhs.z * rhs;
  return tempi;
};

vectori operator *(long int lhs, vectori rhs)
{
  tempi.x = lhs * rhs.x;
  tempi.y = lhs * rhs.y;
  tempi.z = lhs * rhs.z;
  return tempi;
};

bool operator *=(vectorf &lhs, float rhs)
{
  lhs.x *= rhs;
  lhs.y *= rhs;
  lhs.z *= rhs;
  return true;
};

bool operator *=(vectori &lhs, long int rhs)
{
  lhs.x *= rhs;
  lhs.y *= rhs;
  lhs.z *= rhs;
  return true;
};
//=======================================================================
//Quotients

vectorf operator /(vectorf lhs, float rhs)
{
  tempf.x = lhs.x / rhs;
  tempf.y = lhs.y / rhs;
  tempf.z = lhs.z / rhs;
  return tempf;
};

vectori operator /(vectori lhs, long int rhs)
{
  tempi.x = lhs.x / rhs;
  tempi.y = lhs.y / rhs;
  tempi.z = lhs.z / rhs;
  return tempi;
};

bool operator /=(vectorf &lhs, float rhs)
{
  if (rhs == 0.0f)
    return false;
  else
    lhs.x /= rhs;
    lhs.y /= rhs;
    lhs.z /= rhs;
  return true;
};

//========================================================================
//Cross-products (yay!)

vectorf operator &(vectorf lhs, vectorf rhs)
{
  tempf.x = lhs.y * rhs.z - lhs.z * rhs.y;
  tempf.y = lhs.z * rhs.x - lhs.x * rhs.z;
  tempf.z = lhs.x * rhs.y - lhs.y * rhs.x;
  return tempf;
};

vectori operator &(vectori lhs, vectori rhs)
{
  tempi.x = lhs.y * rhs.z - lhs.z * rhs.y;
  tempi.y = lhs.z * rhs.x - lhs.x * rhs.z;
  tempi.z = lhs.x * rhs.y - lhs.y * rhs.x;
  return tempi;
};

//=========================================================================
//projections.

vectorf operator <<(vectorf lhs, vectorf rhs)
//rhs gets projected onto lhs.
{
  return ((lhs * rhs) * lhs) / (lhs * lhs);
};

bool operator <= (vectorf &lhs, vectorf rhs)
{
  if (snorm(lhs) == 0.0f)
    return false;
  else
    {
      lhs = ((lhs * rhs) * lhs) / (lhs * lhs);
      return true;
    };
};

vectori operator <<(vectori lhs, vectori rhs)
{
  return ((lhs * rhs) * lhs) / (lhs * lhs);
};

//=========================================================================
//Norm

float norm(vectorf lhs)
{
  return sqrt(lhs * lhs);
};

float snorm (vectorf lhs)
{
  return lhs * lhs;
};

vectorf unit(vectorf lhs)
{
  fptemp = norm(lhs);
  if (fptemp != 0.0f)
  {
    tempf.x = tempf.x / fptemp;
    tempf.y = tempf.y / fptemp;
    tempf.z = tempf.z / fptemp;
  }
  else {tempf = lhs;};
  
  return tempf;
};