//Vector++.h -- this defines the two major types of vectors:
//  There are floating vectors and integer vectors, object-oriented-style.

#ifndef VECTOR_PLUSPLUS_H
#define VECTOR_PLUSPLUS_H

  #if BUILDING_VECTOR_PLUSPLUS_H
    #define VECTOR_IMPEXP_TYPED __declspec(dllexport)
  #else
    #define VECTOR_IMPEXP_TYPED __declspec(dllimport)
  #endif

class VECTOR_IMPEXP_TYPED vectorf
{
  public:
    vectorf();
    vectorf(float, float, float);
    vectorf(vectorf *);
    float x, y, z;
};  //three-component floating-point vector.

class vectori
{
  public:
    vectori();
    vectori(long int, long int, long int);
    vectori(vectori *);
    long int x, y, z;
};  //three-component long integer vector.

//==========================================================
//sums and differences

VECTOR_IMPEXP_TYPED vectorf operator +(vectorf, vectorf);
VECTOR_IMPEXP_TYPED vectori operator +(vectori, vectori);

VECTOR_IMPEXP_TYPED bool operator +=(vectorf &, vectorf);
VECTOR_IMPEXP_TYPED bool operator +=(vectori &, vectori);

VECTOR_IMPEXP_TYPED vectorf operator -(vectorf, vectorf);
VECTOR_IMPEXP_TYPED vectori operator -(vectori, vectori);

VECTOR_IMPEXP_TYPED bool operator -=(vectorf &, vectorf);
VECTOR_IMPEXP_TYPED bool operator -=(vectori &, vectori);

VECTOR_IMPEXP_TYPED vectorf operator +(vectorf);			//unary-additive
VECTOR_IMPEXP_TYPED vectorf operator -(vectorf);
VECTOR_IMPEXP_TYPED vectori operator +(vectori);
VECTOR_IMPEXP_TYPED vectori operator -(vectori);

//==========================================================
//products

VECTOR_IMPEXP_TYPED float operator *(vectorf, vectorf);		//dot-products
VECTOR_IMPEXP_TYPED long int operator *(vectori, vectori);

VECTOR_IMPEXP_TYPED vectorf operator *(vectorf, float);		//scalar products
VECTOR_IMPEXP_TYPED vectorf operator *(float, vectorf);
VECTOR_IMPEXP_TYPED vectori operator *(vectori, long int);
VECTOR_IMPEXP_TYPED vectori operator *(long int, vectori);

VECTOR_IMPEXP_TYPED bool operator *=(vectorf &, float);
VECTOR_IMPEXP_TYPED bool operator *=(vectori &, long int);

VECTOR_IMPEXP_TYPED vectorf operator /(vectorf, float);		//scalar quotients
VECTOR_IMPEXP_TYPED vectori operator /(vectori, long int);

VECTOR_IMPEXP_TYPED bool operator /=(vectorf &, float);
VECTOR_IMPEXP_TYPED bool operator /=(vectori &, long int);

VECTOR_IMPEXP_TYPED vectorf operator &(vectorf, vectorf);	//cross-products
VECTOR_IMPEXP_TYPED vectori operator &(vectori, vectori);

//==========================================================
//Miscellaneous

VECTOR_IMPEXP_TYPED vectorf operator <<(vectorf, vectorf);  //projections
VECTOR_IMPEXP_TYPED vectori operator <<(vectori, vectori);

VECTOR_IMPEXP_TYPED bool operator <=(vectorf &, vectorf);	//projection assignment

VECTOR_IMPEXP_TYPED float norm(vectorf);
VECTOR_IMPEXP_TYPED float snorm(vectorf); 					//squared norm (fast)

VECTOR_IMPEXP_TYPED vectorf unit(vectorf);

#endif